@@ -44,9 +44,13 @@ package Mail::SpamAssassin::AutoWhitelist;
 use strict;
 use warnings;
 use bytes;
+use re 'taint';
+
+use NetAddr::IP 4.000;   # qw(:upper);
 
 use Mail::SpamAssassin;
 use Mail::SpamAssassin::Logger;
+use Mail::SpamAssassin::Util qw(untaint_var);
 
 use vars	qw{
   	@ISA
@@ -61,28 +65,32 @@ sub new {
   $class = ref($class) || $class;
   my ($main, $msg) = @_;
 
+  my $conf = $main->{conf};
   my $self = {
-    'main'		=> $main,
+    main          => $main,
+    factor        => $conf->{auto_whitelist_factor},
+    ipv4_mask_len => $conf->{auto_whitelist_ipv4_mask_len},
+    ipv6_mask_len => $conf->{auto_whitelist_ipv6_mask_len},
   };
 
-  $self->{factor} = $main->{conf}->{auto_whitelist_factor};
-
   my $factory;
   if ($main->{pers_addr_list_factory}) {
     $factory = $main->{pers_addr_list_factory};
   }
   else {
-    my $type = $main->{conf}->{auto_whitelist_factory};
+    my $type = $conf->{auto_whitelist_factory};
     if ($type =~ /^([_A-Za-z0-9:]+)$/) {
-      $type = $1;
+      $type = untaint_var($type);
       eval '
   	    require '.$type.';
             $factory = '.$type.'->new();
-           ';
-      if ($@) { 
-	warn "auto-whitelist: $@";
+            1;
+           '
+      or do {
+	my $eval_stat = $@ ne '' ? $@ : "errno=$!";  chomp $eval_stat;
+	warn "auto-whitelist: $eval_stat\n";
 	undef $factory;
-      }
+      };
       $main->set_persistent_address_list_factory($factory) if $factory;
     }
     else {
@@ -102,7 +110,7 @@ sub new {
 
 ###########################################################################
 
-=item $meanscore = awl->check_address($addr, $originating_ip);
+=item $meanscore = awl->check_address($addr, $originating_ip, $signedby);
 
 This method will return the mean score of all messages associated with the
 given address, or undef if the address hasn't been seen before.
@@ -112,7 +120,7 @@ If B<$originating_ip> is supplied, it will be used in the lookup.
 =cut
 
 sub check_address {
-  my ($self, $addr, $origip) = @_;
+  my ($self, $addr, $origip, $signedby) = @_;
 
   if (!defined $self->{checker}) {
     return undef;		# no factory defined; we can't check
@@ -121,30 +129,38 @@ sub check_address {
   $self->{entry} = undef;
 
   my $fulladdr = $self->pack_addr ($addr, $origip);
-  $self->{entry} = $self->{checker}->get_addr_entry ($fulladdr);
+  my $entry = $self->{checker}->get_addr_entry ($fulladdr, $signedby);
+  $self->{entry} = $entry;
 
-  if (!defined $self->{entry}->{count} || $self->{entry}->{count} == 0) {
+  if (!$entry->{count}) {
     # no entry found
     if (defined $origip) {
       # try upgrading a default entry (probably from "add-addr-to-foo")
       my $noipaddr = $self->pack_addr ($addr, undef);
-      my $noipent = $self->{checker}->get_addr_entry ($noipaddr);
+      my $noipent = $self->{checker}->get_addr_entry ($noipaddr, undef);
 
       if (defined $noipent->{count} && $noipent->{count} > 0) {
 	dbg("auto-whitelist: found entry w/o IP address for $addr: replacing with $origip");
 	$self->{checker}->remove_entry($noipent);
-        # Now assign proper entry the count and totscore values of the no ip entry
-        # instead of assigning the whole value to avoid wiping out any information added
-        # to the previous entry.
-	$self->{entry}->{count} = $noipent->{count};
-	$self->{entry}->{totscore} = $noipent->{totscore};
+        # Now assign proper entry the count and totscore values of the
+        # no-IP entry instead of assigning the whole value to avoid
+        # wiping out any information added to the previous entry.
+	$entry->{count} = $noipent->{count};
+	$entry->{totscore} = $noipent->{totscore};
       }
     }
   }
 
-  if ($self->{entry}->{count} == 0) { return undef; }
+  if ($entry->{count} < 0 ||
+      $entry->{count} != $entry->{count} ||  # test for NaN
+      $entry->{totscore} != $entry->{totscore})
+  {
+    warn "auto-whitelist: resetting bad data for ($addr, $origip), ".
+         "count: $entry->{count}, totscore: $entry->{totscore}\n";
+    $entry->{count} = $entry->{totscore} = 0;
+  }
 
-  return $self->{entry}->{totscore}/$self->{entry}->{count};
+  return !$entry->{count} ? undef : $entry->{totscore} / $entry->{count};
 }
 
 ###########################################################################
@@ -178,6 +194,10 @@ sub add_score {
   if (!defined $self->{checker}) {
     return undef;		# no factory defined; we can't check
   }
+  if ($score != $score) {
+    warn "auto-whitelist: attempt to add a $score to AWL entry ignored\n";
+    return undef;		# don't try to add a NaN
+  }
 
   $self->{entry}->{count} ||= 0;
   $self->{checker}->add_score($self->{entry}, $score);
@@ -193,9 +213,9 @@ This method will add a score of -100 to the given address -- effectively
 =cut
 
 sub add_known_good_address {
-  my ($self, $addr) = @_;
+  my ($self, $addr, $signedby) = @_;
 
-  return $self->modify_address($addr, -100);
+  return $self->modify_address($addr, -100, $signedby);
 }
 
 
@@ -209,30 +229,30 @@ This method will add a score of 100 to the given address -- effectively
 =cut
 
 sub add_known_bad_address {
-  my ($self, $addr) = @_;
+  my ($self, $addr, $signedby) = @_;
 
-  return $self->modify_address($addr, 100);
+  return $self->modify_address($addr, 100, $signedby);
 }
 
 ###########################################################################
 
 sub remove_address {
-  my ($self, $addr) = @_;
+  my ($self, $addr, $signedby) = @_;
 
-  return $self->modify_address($addr, undef);
+  return $self->modify_address($addr, undef, $signedby);
 }
 
 ###########################################################################
 
 sub modify_address {
-  my ($self, $addr, $score) = @_;
+  my ($self, $addr, $score, $signedby) = @_;
 
   if (!defined $self->{checker}) {
     return undef;		# no factory defined; we can't check
   }
 
   my $fulladdr = $self->pack_addr ($addr, undef);
-  my $entry = $self->{checker}->get_addr_entry ($fulladdr);
+  my $entry = $self->{checker}->get_addr_entry ($fulladdr, $signedby);
 
   # remove any old entries (will remove per-ip entries as well)
   # always call this regardless, as the current entry may have 0
@@ -240,10 +260,11 @@ sub modify_address {
   $self->{checker}->remove_entry($entry);
 
   # remove address only, no new score to add
-  if (!defined($score)) { return 1; }
+  if (!defined $score)  { return 1; }
+  if ($score != $score) { return 1; }  # don't try to add a NaN
 
   # else add score. get a new entry first
-  $entry = $self->{checker}->get_addr_entry ($fulladdr);
+  $entry = $self->{checker}->get_addr_entry ($fulladdr, $signedby);
   $self->{checker}->add_score($entry, $score);
 
   return 1;
@@ -260,6 +281,55 @@ sub finish {
 
 ###########################################################################
 
+sub ip_to_awl_key {
+  my ($self, $origip) = @_;
+
+  my $result;
+  local $1;
+  if (!defined $origip) {
+    # could not find an IP address to use
+  } elsif ($origip =~ /^ (\d{1,3} \. \d{1,3}) \. \d{1,3} \. \d{1,3} $/xs) {
+    my $mask_len = $self->{ipv4_mask_len};
+    $mask_len = 16  if !defined $mask_len;
+    # handle the default and easy cases manually
+    if ($mask_len == 32) {
+      $result = $origip;
+    } elsif ($mask_len == 16) {
+      $result = $1;
+    } else {
+      my $origip_obj = NetAddr::IP->new($origip . '/' . $mask_len);
+      if (!defined $origip_obj) {  # invalid IPv4 address
+        dbg("auto-whitelist: bad IPv4 address $origip");
+      } else {
+        $result = $origip_obj->network->addr;
+        $result =~s/(\.0){1,3}\z//;  # truncate zero tail
+      }
+    }
+  } elsif ($origip =~ /:/ &&  # triage
+           $origip =~
+           /^ [0-9a-f]{0,4} (?: : [0-9a-f]{0,4} | \. [0-9]{1,3} ){2,9} $/xsi) {
+    # looks like an IPv6 address
+    my $mask_len = $self->{ipv6_mask_len};
+    $mask_len = 48  if !defined $mask_len;
+    my $origip_obj = NetAddr::IP->new6($origip . '/' . $mask_len);
+    if (!defined $origip_obj) {  # invalid IPv6 address
+      dbg("auto-whitelist: bad IPv6 address $origip");
+    } else {
+      $result = $origip_obj->network->full6;  # string in a canonical form
+      $result =~ s/(:0000){1,7}\z/::/;        # compress zero tail
+    }
+  } else {
+    dbg("auto-whitelist: bad IP address $origip");
+  }
+  if (defined $result && length($result) > 39) {  # just in case, keep under
+    $result = substr($result,0,39);               # the awl.ip field size
+  }
+  dbg("auto-whitelist: IP masking %s -> %s", $origip,$result);
+  return $result;
+}
+
+###########################################################################
+
 sub pack_addr {
   my ($self, $addr, $origip) = @_;
 
@@ -267,15 +337,13 @@ sub pack_addr {
   $addr =~ s/[\000\;\'\"\!\|]/_/gs;	# paranoia
 
   if (!defined $origip) {
-    # could not find an IP address to use, could be localhost mail or from
-    # the user running "add-addr-to-*".
+    # could not find an IP address to use, could be localhost mail
+    # or from the user running "add-addr-to-*".
     $origip = 'none';
   } else {
-    $origip =~ s/\.\d{1,3}\.\d{1,3}$//gs;
+    $origip = $self->ip_to_awl_key($origip);
   }
-
-  $origip =~ s/[^0-9\.noe]/_/gs;	# paranoia
-  $addr."|ip=".$origip;
+  return $addr . "|ip=" . $origip;
 }
 
 ###########################################################################
