@@ -20,17 +20,14 @@ package Mail::SpamAssassin::DBBasedAddrList;
 use strict;
 use warnings;
 use bytes;
+use re 'taint';
 use Fcntl;
 
 use Mail::SpamAssassin::PersistentAddrList;
 use Mail::SpamAssassin::Util;
 use Mail::SpamAssassin::Logger;
 
-use vars qw{
-  @ISA
-};
-
-@ISA = qw(Mail::SpamAssassin::PersistentAddrList);
+our @ISA = qw(Mail::SpamAssassin::PersistentAddrList);
 
 ###########################################################################
 
@@ -63,13 +60,15 @@ sub new_checker {
 	$main->{conf}->{auto_whitelist_db_modules}."\n";
   }
 
+  my $umask = umask ~ (oct($main->{conf}->{auto_whitelist_file_mode}));
+
   # if undef then don't worry -- empty hash!
   if (defined($main->{conf}->{auto_whitelist_path})) {
     my $path = $main->sed_path($main->{conf}->{auto_whitelist_path});
     my ($mod1, $mod2);
 
     if ($main->{locker}->safe_lock
-                       ($path, 30, $main->{conf}->{auto_whitelist_file_mode}))
+            ($path, 30, $main->{conf}->{auto_whitelist_file_mode}))
     {
       $self->{locked_file} = $path;
       $self->{is_locked}   = 1;
@@ -82,16 +81,21 @@ sub new_checker {
 
     dbg("auto-whitelist: tie-ing to DB file of type $dbm_module $mod1 in $path");
 
+    ($self->{is_locked} && $dbm_module eq 'DB_File') and
+            Mail::SpamAssassin::Util::avoid_db_file_locking_bug($path);
+
     if (! tie %{ $self->{accum} }, $dbm_module, $path, $mod2,
-      oct($main->{conf}->{auto_whitelist_file_mode}) ) {
-        my $err = $!;   # might get overwritten later
-        if ($self->{is_locked}) {
-          $self->{main}->{locker}->safe_unlock($self->{locked_file});
-          $self->{is_locked} = 0;
-        }
-        die "auto-whitelist: cannot open auto_whitelist_path $path: $err\n";
+            oct($main->{conf}->{auto_whitelist_file_mode}) & 0666)
+    {
+      my $err = $!;   # might get overwritten later
+      if ($self->{is_locked}) {
+        $self->{main}->{locker}->safe_unlock($self->{locked_file});
+        $self->{is_locked} = 0;
       }
+      die "auto-whitelist: cannot open auto_whitelist_path $path: $err\n";
+    }
   }
+  umask $umask;
 
   bless ($self, $class);
   return $self;
@@ -114,7 +118,7 @@ sub finish {
 ###########################################################################
 
 sub get_addr_entry {
-  my ($self, $addr) = @_;
+  my ($self, $addr, $signedby) = @_;
 
   my $entry = {
 	addr			=> $addr,
