@@ -49,6 +49,7 @@ use Mail::SpamAssassin::Logger;
 use strict;
 use warnings;
 use bytes;
+use re 'taint';
 
 use vars qw(@ISA);
 @ISA = qw(Mail::SpamAssassin::Plugin);
@@ -84,7 +85,7 @@ sub new {
 
 sub set_config {
   my ($self, $conf) = @_;
-  my @cmds = ();
+  my @cmds;
 
 =head1 USER OPTIONS
 
@@ -353,14 +354,19 @@ sub load_models {
   my $rang = 1;
   dbg("textcat: loading languages file...");
 
+  local *LM;
   if (!open(LM, $languages_filename)) {
-    warn "textcat: cannot open languages file: $!\n";
+    warn "textcat: cannot open languages file $languages_filename: $!\n";
     return;
   }
 
-  local $/ = undef;
-  @lm = split(/\n/, <LM>);
-  close(LM);
+  { my($inbuf,$nread,$text); $text = '';
+    while ( $nread=read(LM,$inbuf,16384) ) { $text .= $inbuf }
+    defined $nread  or die "error reading $languages_filename: $!";
+    @lm = split(/\n/, $text, -1);
+  }
+
+  close(LM)  or die "error closing $languages_filename: $!";
   # create language ngram maps once
   for (@lm) {
     # look for end delimiter
