@@ -58,7 +58,7 @@ BEGIN {
   @ISA = qw(Exporter);
   @EXPORT = ();
   @EXPORT_OK = qw(&local_tz &base64_decode &untaint_var &untaint_file_path
-                  &exit_status_str &proc_status_ok);
+                  &exit_status_str &proc_status_ok &am_running_on_windows);
 }
 
 use Mail::SpamAssassin;
@@ -329,7 +329,7 @@ sub exit_status_str($;$) {
   } else {
     my $sig = WTERMSIG($stat);
     $str = sprintf("%s, signal %d (%04x)",
-             $sig == 1 ? 'HANGUP' : $sig == 2 ? 'INTERRUPTED' :
+             $sig == 1 ? 'HANGUP' : $sig == 2 ? 'interrupted' :
              $sig == 6 ? 'ABORTED' : $sig == 9 ? 'KILLED' :
              $sig == 15 ? 'TERMINATED' : 'DIED',
              $sig, $stat);
@@ -1555,8 +1555,9 @@ sub helper_app_pipe_open_unix {
 
 sub trap_sigalrm_fully {
   my ($handler) = @_;
-  if ($] < 5.008) {
-    # signals are always unsafe, just use %SIG
+  if ($] < 5.008 || am_running_on_windows()) {
+    # signals are always unsafe on perl older than 5.008, just use %SIG
+    # Bug 6359, no POSIX::SIGALRM on Windows, just use %SIG
     $SIG{ALRM} = $handler;
   } else {
     # may be using "safe" signals with %SIG; use POSIX to avoid it
